package jalview.ws.jws2;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import compbio.metadata.Argument;
import compbio.metadata.Option;
import compbio.metadata.Parameter;
import compbio.metadata.Preset;
import compbio.metadata.PresetManager;
import compbio.metadata.RunnerConfig;

import jalview.ws.jws2.Jws2Discoverer.Jws2Instance;
import jalview.ws.jws2.dm.JabaOption;
import jalview.ws.jws2.dm.JabaParameter;
import jalview.ws.jws2.dm.JabaWsParamSet;
import jalview.ws.params.ArgumentI;
import jalview.ws.params.ParamDatastoreI;
import jalview.ws.params.WsParamSetI;

public class JabaParamStore implements ParamDatastoreI
{

  Hashtable<String, JabaWsParamSet> editedParams = new Hashtable<String, JabaWsParamSet>();

  private Jws2Instance service;

  private RunnerConfig serviceOptions;

  private Hashtable<String, JabaPreset> servicePresets;

  public JabaParamStore(Jws2Instance service)
  {
    this.service = service;
    serviceOptions = service.getRunnerConfig();
    // TODO: discover all the user's locally sored presets for this service and populate the hash table
    // TODO: for JAL-632
  }

  /**
   * store the given parameters in the user parameter set database.
   * 
   * @param storeSetName
   *          - lastParmSet
   * @param descr
   *          - setDescr.getText()
   * @param jobParams
   *          - getJobParams()
   */
  private void _storeUserPreset(String storeSetName, String descr,
          List<Argument> jobParams)
  {
    // this is a simple hash store.
    editedParams.put(storeSetName, new JabaWsParamSet(storeSetName, descr,
            JabaParamStore.getJwsArgsfromJaba(jobParams)));
    // writeParam("Saving " + storeSetName + ": ", jobParams);
  }

  private void writeParam(String nm, List<Argument> params)
  {
    for (Argument p : params)
    {
      System.out.println(nm + ":" + System.identityHashCode(p) + " Name: "
              + p.getName() + " Value: " + p.getDefaultValue());
    }
  }

  private JabaWsParamSet _getUserPreset(String setName)
  {
    JabaWsParamSet pset = editedParams.get(setName);
    return pset;
  }

  /**
   * remove the given user preset from the preset stash
   * 
   * @param setName
   */
  private void _deleteUserPreset(String setName)
  {
    editedParams.remove(setName);
  }

  @Override
  public List<WsParamSetI> getPresets()
  {
    List<WsParamSetI> prefs = new ArrayList();
    if (servicePresets == null)
    {
      servicePresets = new Hashtable<String, JabaPreset>();
      PresetManager prman;
      if ((prman = service.getPresets()) != null)
      {
        List pset = prman.getPresets();
        if (pset != null)
        {
          for (Object pr : pset)
          {
            JabaPreset prset = new JabaPreset(service, (Preset) pr);
            servicePresets.put(prset.getName(), prset);
          }
        }
      }
    }
    for (JabaPreset pr : servicePresets.values())
    {
      prefs.add(pr);
    }
    for (WsParamSetI wspset : editedParams.values())
    {
      prefs.add(wspset);
    }
    return prefs;
  }

  @Override
  public WsParamSetI getPreset(String name)
  {
    for (WsParamSetI pr : getPresets())
    {
      if (pr.getName().equals(name))
      {
        return pr;
      }
    }
    return null;
  }

  public static List<ArgumentI> getJwsArgsfromJaba(List jabargs)
  {
    List<ArgumentI> rgs = new ArrayList<ArgumentI>();
    for (Object rg : jabargs)
    {
      ArgumentI narg = (rg instanceof Parameter) ? new JabaParameter(
              (Parameter) rg) : (rg instanceof Option) ? new JabaOption(
              (Option) rg) : null;
      if (narg == null)
      {
        throw new Error(
                "Implementation Error: Cannot handle Jaba parameter object "
                        + rg.getClass());
      }
      else
      {
        rgs.add(narg);
      }
    }
    return rgs;
  }

  public static List getJabafromJwsArgs(List<ArgumentI> jwsargs)
  {
    List rgs = new ArrayList();
    for (ArgumentI rg : jwsargs)
    {
      Argument narg = (rg instanceof JabaOption) ? ((JabaOption) rg)
              .getOption() : null;
      if (narg == null)
      {
        throw new Error(
                "Implementation Error: Cannot handle Jaba parameter object "
                        + rg.getClass());
      }
      else
      {
        rgs.add(narg);
      }
    }
    return rgs;
  }

  @Override
  public List<ArgumentI> getServiceParameters()
  {
    return getJwsArgsfromJaba(serviceOptions.getArguments());
  }

  @Override
  public boolean presetExists(String name)
  {
    return (editedParams.containsKey(name) || servicePresets
            .containsKey(name));
  }

  @Override
  public void deletePreset(String name)
  {
    if (editedParams.containsKey(name))
    {
      editedParams.remove(name);
      return;
    }
    if (servicePresets.containsKey(name))
    {
      throw new Error(
              "Implementation error: Attempt to delete a service preset!");
    }
  }


  @Override
  public void storePreset(String presetName, String text,
          List<ArgumentI> jobParams)
  {
    JabaWsParamSet jps = new JabaWsParamSet(presetName, text, jobParams);
    editedParams.put(jps.getName(), jps);
  }

  @Override
  public boolean readParamSet(WsParamSetI wsp,
          Reader reader) throws IOException
  {
    if (!(wsp instanceof JabaWsParamSet))
    {
      throw new Error("Implementation error: JabaWsParamSets can only be handled by JabaParamStore");
    }
    List<String> lines=new ArrayList<String>();
    String line;
    BufferedReader br = new BufferedReader(reader);
    while ((line=br.readLine())!=null)
    {
      lines.add(line);
    }
    ((JabaWsParamSet) wsp).setjabaArguments(ParameterUtils.processParameters(lines, serviceOptions, " "));
    return true;
  }

  @Override
  public boolean writeParamSet(WsParamSetI wsp,
          Writer writer) throws IOException
  {
            if (!(wsp instanceof JabaWsParamSet))
            {
              throw new Error("Implementation error: JabaWsParamSets can only be handled by JabaParamStore");
            }
            writer.write(ParameterUtils.writeParameterSet(((JabaWsParamSet)wsp).getjabaArguments(), " ")
                    .toString());
            return true;
  }

}
